/*
======================================================
rslib component:  bitstream prototypes
======================================================
*/

#ifndef bitstream_h
#define bitstream_h



namespace rslib
{

	

    class BitStream
    {		
		protected:
			typedef unsigned long int bsint;
			
			//deque<bool> stream;		//temp, so it can compile - until link list conversion done
			
			
			//a node holds one bit  (would be more space efficient to hold a char or something, and store 8 bits, but require more processing)
			class Node
			{
				public:
					Node();
					Node(Node * prevN,Node * nextN);
					Node * prev;
					Node * next;
					bool data;
			};
			
			Node * m_start;			//first node
			Node * m_end;			//last node
			bsint m_bitsize;			//size in bits
			
			
			//INTERNAL FUNCTIONS:
			//------------------
			
			//node related functions:
			Node * AddNode(bool start=false);	//add node at start or end of array and then return this new node, default: add to end
			Node * InsertNodeAfter(Node * ref);		//Inserts a node after reference node
			Node * InsertNodeBefore(Node * ref);		//Inserts a node before reference node			
			void DelNode(Node * todel);		//delete node
			Node * GetNextNode(Node * ref);	//get next node after ref
			Node * GetPrevNode(Node * ref);	//get previous node after ref
			Node * GetNode(bsint pos);		//get node at given pos
			
			void WriteBit_internal(bool tw,Node * ref,bool start=false);	//writes a bit after reference, if null it writes at end by default, or start if start=true
			void WriteVarInt_internal(bsint tw,int bits,Node * ref,bool start=false);//same as above but with a var int			
			void WriteString_internal(string tw,Node * ref,bool start=false); //same as above but with string
			void WriteBitStream_internal(BitStream tw,Node * ref,bool start=false); //same as above but with BitStream			
			bsint ReadVarInt_internal(int bits,Node * ref,bool start=false,bool pop=false);	//similar as above, but reading var int
			string ReadString_internal(bsint chars,Node * ref,bool start=false,bool pop=false);//same as above but with string
			BitStream ReadBitStream_internal(bsint bits,Node * ref,bool start=false,bool pop=false);//same as above but with string
			
			
			//misc stuff:
			//-----------
			
			#ifdef rslib_pthread
				MUTEX mutex;
			#endif		
    
            
        public: 
			BitStream();
			BitStream(const BitStream &copy); //copy constructor
			BitStream(const string& copy); //copy constructor
			~BitStream();
			bool Lock(bool block);		//attempt to lock class for i/o - blocks if its already locked, unless block=false (will return false on fail)
			void Unlock();				//unlock class for i/o
			
			string ToStr();						//converts to string 
			string ToHex(int groupnum=1,bool showx=true);						//stream of HEX digits
			string ToBitStr(bool bytespaces);	//converts to string of 1/0's grouped in 8
			string HexDump(unsigned int bytesperline=26);				//dumps hex/ascii
			void DisplayInfo();        	//displays information
			void Display();				//displays it in bits
			void Clear();				//erases everything
			bsint Size();				//size in bits (ex: a 2 char string = size of 16)
			bool IsEmpty();				//returns true if empty
			
			void DeleteBits(int bits,bsint pos);	//deletes bits at specified position
			
			//obsolete:
			//void ReverseBits(int amt,bsint pos); //reverses the bits of the chosen block
			//void ReverseBytes(int amt,bsint pos); //reverses the bytes of the chosen block (good for making endianess easier to deal with)
			
			
			//writers: 
			//inserts data into specified position  (data is written BEFORE the position bit)
			
			void WriteBit(bool towrite,bsint pos);
			void WriteVarInt(bsint towrite,int bitstouse,bsint pos);        
			void WriteInt8(bsint towrite,bsint pos);
			void WriteInt16(bsint towrite,bsint pos);
			void WriteInt24(bsint towrite,bsint pos);
			void WriteInt32(bsint towrite,bsint pos);
			void WriteString(string towrite,bsint pos);
			void WriteBitStream(BitStream towrite,bsint pos);
			
			//writer overloads for pushing:
			void WriteBit(bool towrite);
			void WriteVarInt(bsint towrite,int bitstouse);        
			void WriteInt8(bsint towrite);
			void WriteInt16(bsint towrite);
			void WriteInt24(bsint towrite);
			void WriteInt32(bsint towrite);
			void WriteString(string towrite);
			void WriteBitStream(BitStream towrite);
		
			//readers:
			//reads data at specified position
			bool ReadBit(bsint pos);	
			bsint ReadVarInt(bsint pos,int bits);
			bsint ReadInt8(bsint pos);
			bsint ReadInt16(bsint pos);
			bsint ReadInt24(bsint pos);
			bsint ReadInt32(bsint pos);
			string ReadString(bsint pos,int chars);
			BitStream ReadBitStream(bsint pos,int bits);
			
			
			//poppers:
			//pops data from start or end, removing it from the array
			
			bool PopBit_s(); //pop bit from start
			bool PopBit_e(); //pop bit from end

			bsint PopIntVar_s(int bits);//pop 1-32 bits from start
			bsint PopIntVar_e(int bits);//pop 1-32 bits from end

			bsint PopInt8_s();//pop byte (as int) from start
			bsint PopInt8_e();//pop byte (as int) from end
			
			bsint PopInt16_s();//pop 16 bits from start
			bsint PopInt16_e();//pop 16 bits from end
			
			bsint PopInt24_s();//pop 24 bits from start
			bsint PopInt24_e();//pop 24 bits from end
			
			bsint PopInt32_s();//pop 32 bits from start
			bsint PopInt32_e();//pop 32 bits from end
			
			string PopString_s(bsint chars);//pop string from start
			string PopString_e(bsint chars);//pop string from end
			
			BitStream PopBitStream_s(bsint bits);//pop string from start
			BitStream PopBitStream_e(bsint bits);//pop string from end
			
			
			//pushers:
			//pushes data to start or end of array
			
			void PushBit_s(bool tw); //push bit at start
			void PushBit_e(bool tw); //push bit at end
			
			void PushIntVar_s(bsint tw,int bits); //push var int at start
			void PushIntVar_e(bsint tw,int bits); //push var int at end
			
			void PushInt8_s(bsint  tw); //push stuff to start
			void PushInt8_e(bsint  tw); //push stuff to end
			
			void PushInt16_s(bsint  tw); //push stuff to start
			void PushInt16_e(bsint  tw); //push stuff to end
			
			void PushInt24_s(bsint  tw); //push stuff to start
			void PushInt24_e(bsint  tw); //push stuff to end
			
			void PushInt32_s(bsint  tw); //push stuff to start
			void PushInt32_e(bsint  tw); //push stuff to end
			
			void PushString_s(string tw);			//push string to start
			void PushString_e(string tw);			//push string to end

			void PushBitStream_s(BitStream tw);		//push bitstream to start
			void PushBitStream_e(BitStream tw);		//push bitstream to end        

			
			//overloads:
						
			BitStream& operator=(BitStream& other);	
			BitStream operator=(const BitStream& other);	
			
			BitStream& operator=(string& other);		
			BitStream operator=(const string& other);
			


			BitStream& operator+=(BitStream&);
			BitStream& operator+=(string);
			
			BitStream operator+(const BitStream &other) const;		
			BitStream operator+(const string &other) const;	
			BitStream operator+(const char*& other) const;	
			
			bool operator==( BitStream& tw);
			bool operator==( string tw);
			

			//statics:			
				
			static int MaxBits(); 
			static int MaxBitsOf(bsint a); 
			static deque<bool> Int2Bin(bsint tc);    
			static string Int2BinStr(bsint tc); 
			static BitStream Int2BinBS(bsint tc);
			static string Int2Str(bsint tc);
			static string Int2HexStr(bsint tc,bool dontshowx);

			static string Double2Str(long double tc);		

			static string Str2HexStr(string tc,int groupnum,bool showx);
			static bsint Str2Int(string tc);	
        
        
        
    };

}


#endif
