/*
======================================================
rslib component:  ByteStream prototypes
======================================================
*/

#ifndef ByteStream_h
#define ByteStream_h



namespace rslib
{

	

    class ByteStream
    {		
		public:
			typedef unsigned long int bsint;
		
		protected:			
						
			
			//a node holds one byte
			class Node
			{
				public:
					Node();
					Node(Node * prevN,Node * nextN);
					Node * prev;
					Node * next;
					unsigned char data;
			};
			
			Node * m_start;			//first node
			Node * m_end;			//last node
			uli m_size;			//size in bits
			
			
			//INTERNAL FUNCTIONS:
			//------------------

			bool EnsureMaxIntSize(uli & tw,int bytes);	//ensures tw is not bigger then what "bytes" can handle
			bool EnsureMaxPos(uli & pos,uli & bytes);	//ensures there are "bytes" bytes after position (including position itself)		
			
			//node related functions:
			Node * AddNode(bool start=false);	//add node at start or end of array and then return this new node, default: add to end
			Node * InsertNodeAfter(Node * ref);		//Inserts a node after reference node
			Node * InsertNodeBefore(Node * ref);		//Inserts a node before reference node
			void DelNode(Node * todel);		//delete node
			Node * GetNextNode(Node * ref);	//get next node after ref
			Node * GetPrevNode(Node * ref);	//get previous node after ref
			Node * GetNode(bsint pos);		//gets a node
			
			
			//reads/writes data after reference, if null it writes at end by default, or start if start=true: 				

			void WriteVarInt_internal(uli tw,int bytes, Node * ref, bool start=false);	//writes var int 
			void WriteString_internal(string tw,Node * ref,bool start=false); //writes string
			void WriteByteStream_internal(ByteStream tw,Node * ref,bool start=false); //writes bytestream
			
			uli ReadVarInt_internal(int bytes,Node * ref,bool start=false,bool pop=false);//reads string
			string ReadString_internal(bsint chars,Node * ref,bool start=false,bool pop=false);//reads string
			ByteStream ReadByteStream_internal(bsint bytes,Node * ref,bool start=false,bool pop=false);//reads bytestream
			
			
			//misc stuff:
			//-----------
			
			#ifdef rslib_pthread
				MUTEX mutex;
			#endif		
    
            
        public: 
			ByteStream();
			ByteStream(const ByteStream &copy); //copy constructor for bytestream		
			ByteStream(const string& copy); //copy constructor for string
			
			~ByteStream();
			bool Lock(bool block);		//attempt to lock class for i/o - blocks if its already locked, unless block=false (will return false on fail)
			void Unlock();				//unlock class for i/o
			
			string ToStr();						//converts to string 
			string ToHex(int groupnum=1,bool showx=true);						//stream of HEX digits
			string HexDump(unsigned int bytesperline=26);				//dumps hex/ascii
			void DisplayInfo();        	//displays information
			void Display();				//displays it in bytes and hex
			void Clear();				//erases everything
			bsint Size();				//size in bytes
			bool IsEmpty();				//returns true if empty

			

			//writers: 
			//inserts data into specified position  (data is written BEFORE the position byte)			
			
			void WriteInt8(bsint towrite,bsint pos);
			void WriteInt16(bsint towrite,bsint pos);
			void WriteInt24(bsint towrite,bsint pos);
			void WriteInt32(bsint towrite,bsint pos);
			void WriteString(string towrite,bsint pos);
			void WriteByteStream(ByteStream towrite,bsint pos);
			
			//writer overloads for pushing:      
			void WriteInt8(bsint towrite);
			void WriteInt16(bsint towrite);
			void WriteInt24(bsint towrite);
			void WriteInt32(bsint towrite);
			void WriteString(string towrite);
			void WriteByteStream(ByteStream towrite);
		
			//readers:
			//reads data at specified position
			bsint ReadVarInt(bsint pos,int bytes);		
			bsint ReadInt8(bsint pos);
			bsint ReadInt16(bsint pos);
			bsint ReadInt24(bsint pos);
			bsint ReadInt32(bsint pos);
			string ReadString(bsint pos,bsint chars);
			ByteStream ReadByteStream(bsint pos,bsint bytes);
			
			
			//poppers:
			//pops data from start or end, removing it from the array



			bsint PopInt8_s();//pop byte (as int) from start
			bsint PopInt8_e();//pop byte (as int) from end
			
			bsint PopInt16_s();//pop 16 bits from start
			bsint PopInt16_e();//pop 16 bits from end
			
			bsint PopInt24_s();//pop 24 bits from start
			bsint PopInt24_e();//pop 24 bits from end
			
			bsint PopInt32_s();//pop 32 bits from start
			bsint PopInt32_e();//pop 32 bits from end
			
			string PopString_s(bsint chars);//pop string from start
			string PopString_e(bsint chars);//pop string from end
			
			ByteStream PopByteStream_s(bsint bits);//pop string from start
			ByteStream PopByteStream_e(bsint bits);//pop string from end
			
			
			//pushers:
			//pushes data to start or end of array
			
			void PushInt8_s(bsint  tw); //push stuff to start
			void PushInt8_e(bsint  tw); //push stuff to end
			
			void PushInt16_s(bsint  tw); //push stuff to start
			void PushInt16_e(bsint  tw); //push stuff to end
			
			void PushInt24_s(bsint  tw); //push stuff to start
			void PushInt24_e(bsint  tw); //push stuff to end
			
			void PushInt32_s(bsint  tw); //push stuff to start
			void PushInt32_e(bsint  tw); //push stuff to end
			
			void PushString_s(string tw);			//push string to start
			void PushString_e(string tw);			//push string to end

			void PushByteStream_s(ByteStream tw);		//push ByteStream to start
			void PushByteStream_e(ByteStream tw);		//push ByteStream to end        

			
			//statics and overloads:
			
			static int MaxBytes();			//displays max bytes that can be worked with
			
						
			ByteStream& operator=(ByteStream& other);	
			ByteStream operator=(const ByteStream& other);				
			
			ByteStream& operator=(string& other);		
			ByteStream operator=(const string& other);

			ByteStream& operator+=(ByteStream&);
			ByteStream& operator+=(string);
			
			ByteStream operator+(const ByteStream &other) const;		
			ByteStream operator+(const string &other) const;	
			ByteStream operator+(const char*& other) const;	
			
			bool operator==( ByteStream& tw);
			bool operator==( string tw);
			
		
        
        
        
    };

}


#endif
