/*
======================================================
rslib component: date/time prototypes
======================================================
*/


#ifndef datetime_h
#define datetime_h


namespace rslib
{
	class TimeSpan;
	
	class DateTime
	{
		private:

			//used to emulate the tm struct since tm stuct is static
			int tm_sec;
			int tm_min;
			int tm_hour;
			int tm_mday;
			int tm_mon;
			int tm_year;
			int tm_wday;
			int tm_yday;
			int tm_isdst;
			//----
			
			#ifdef rslib_pthread
				MUTEX tmmutex; //used to control access to TM
			#endif
			
			time_t m_time; //the actual time
			string m_format; //date format, uses same format as php date()
			
			
			void RebuildTime(); //regenerates m_time based on m_timesplit data
			void RebuildTS(); //regenerates m_timesplit based on m_time data

		public:
			DateTime();
			DateTime(time_t setto);
			void SetFormat(string format);		//sets date format
			string GetDate(string format);	//returns date with a specific format
			string GetDate();							//get local date with default format
			
			void SetYear(int year); //sets year 
			void SetMonth(int month); //sets month (1-12)
			void SetDay(int day); //sets day of month (1-31)
			void SetHour(int hour); //sets hour (0-23)
			void SetMinute(int minute); //sets minute (0-59)
			void SetSecond(int second); //sets second (0-59)
			
			
			void AddSeconds(int secs); 		//add seconds to date
			void AddMinutes(int mins); 		//add minutes to date
			void AddHours(int hours); 		//add hours to date
			void AddDays(int days); 		//add days to date

			
			void SetNow();				//sets the date/time right now
			
			
			//overloads:
			DateTime & operator=(const DateTime &eq);
			DateTime & operator+=(const TimeSpan &add);
			DateTime & operator-=(const TimeSpan &add);			
			const DateTime operator+(const TimeSpan &add) const;
			const DateTime operator-(const TimeSpan &add) const;	
			bool operator==(const DateTime &other) const;
			bool operator!=(const DateTime &other) const;
			bool operator>=(const DateTime &other) const;
			bool operator>(const DateTime &other) const;
			bool operator<=(const DateTime &other) const;
			bool operator<(const DateTime &other) const;

			
			
			//STATICS:
			//----------
				
			static DateTime Now();		//returns datetime with current time
	
			//formatters:
			static string GDF_d(int val); //day of the month with leading zeros
			static string GDF_D(int val); //A textual representation of a day, three letters
			static string GDF_j(int val); //Day of the month without leading zeros
			static string GDF_l(int val); //A full textual representation of the day of the week
			static string GDF_N(int val); //ISO-8601 numeric representation of the day of the week
			static string GDF_S(int val); //English ordinal suffix for the day of the month, 2 characters
			static string GDF_w(int val); //Numeric representation of the day of the week
			static string GDF_z(int val); //The day of the year (starting from 0)
			static string GDF_F(int val); //A full textual representation of a month, such as January or March
			static string GDF_m(int val); //Numeric representation of a month, with leading zeros
			static string GDF_M(int val); //A short textual representation of a month, three letters
			static string GDF_n(int val); //Numeric representation of a month, without leading zeros
			static string GDF_t(int year, int month); //Number of days in the given month
			static string GDF_L(int val); //Whether it's a leap year
			static string GDF_Y(int val); //A full numeric representation of a year, 4 digits
			static string GDF_y(int val); //A two digit representation of a year
			static string GDF_a(int val); //Lowercase Ante meridiem and Post meridiem
			static string GDF_A(int val); //Uppercase Ante meridiem and Post meridiem
			static string GDF_g(int val); //12-hour format of an hour without leading zeros
			static string GDF_G(int val); //24-hour format of an hour with leading zeros
			static string GDF_h(int val); //12-hour format of an hour without leading zeros
			static string GDF_H(int val); //24-hour format of an hour with leading zeros
			static string GDF_i(int val); //Minutes with leading zeros
			static string GDF_s(int val); //Seconds, with leading zeros
			static string GDF_I(int val); //Whether or not the date is in daylight saving time
			static string GDF_U(long int val); //Seconds since the Unix Epoch (January 1 1970 00:00:00 GMT)
	
	
	
	
	
	};
	

	
	
	class TimeSpan
	{
		friend class DateTime;
		
		private:
			long int m_seconds;
			long int m_minutes;
			long int m_hours;
			long int m_days;
			
		
			void Defrag(); //ensures times are displayed properly (ex: 130sec = 2:10 min)			
			
		public:
			TimeSpan();
					
			void SetSeconds(long int seconds);
			void SetMinutes(long double minutes);
			void SetHours(long double hours);
			void SetDays(long double days);	
			
			void AddSeconds(long int seconds);
			void AddMinutes(long double minutes);
			void AddHours(long double hours);
			void AddDays(long double days);					

			string ToStr();
			
			TimeSpan & operator=(const TimeSpan &eq);
			TimeSpan & operator+=(const TimeSpan &add);
			TimeSpan & operator-=(const TimeSpan &add);			
			const TimeSpan operator+(const TimeSpan &add) const;
			const TimeSpan operator-(const TimeSpan &add) const;
			bool operator==(const TimeSpan &other) const;
			bool operator!=(const TimeSpan &other) const;
			bool operator>=(const TimeSpan &other) const;
			bool operator>(const TimeSpan &other) const;
			bool operator<=(const TimeSpan &other) const;
			bool operator<(const TimeSpan &other) const;
			
			
			
			
			
			//static:
			static TimeSpan FromSeconds(long int seconds);
			static TimeSpan FromMinutes(long double minutes);
			static TimeSpan FromHours(long double hours);
			static TimeSpan FromDays(long double days);
			static TimeSpan Zero();
				
	
	};


	
	/*
	Note: In windows it is only precise to the second - should be looked at eventually, may even be off by 1 second at times
	*/
	
	
	class StopWatch
	{
		private:
			#ifdef WIN32
				time_t tt;
				unsigned long int t1,t2;
			#else
				timeval tim;
				double t1,t2;
			#endif
			
			
			bool started;
			
			
		public:
			StopWatch();
			void Start();
			void Stop();
			unsigned long int MiSec();
			unsigned long int MSec();
			unsigned long int Sec();
			void Reset();
	};
}



#endif

