/*
======================================================
rslib component: networking prototypes
======================================================
*/


//Note: To use these ensure to add #define rslib_network before including rslib.h



#ifndef network_h
#define network_h



#ifdef WIN32
#define EWOULDBLOCK WSAEWOULDBLOCK
#endif

namespace rslib
{
        
    //Client stuff:
    //-----------------------------------
    
    class SocketClient
    {
    private:
        string ip;
        int port;
        int status;    
		bool m_blockmode;
		int m_contimeoutsec;
		int m_contimeoutusec;
		string m_lasterror; 
    	
        //socket vars
        int sockfd; 
        struct sockaddr_in dest_addr;  
    	struct hostent *he;    
		
		//RX stuff:
		int rxret; //used for receive return value
		char * rxbuffer; //rx buffer
		int rxbuffersize; //size of rxbuffer
    
    public:
        SocketClient(string,int);
		~SocketClient();
		void SetBlockingMode(bool value); //set to blocking socket
		void SetConnectTimeout(int,int); //set timeout in seconds, usecs
		void SetBufferSize(int psize); //sets size of internal buffer for send/recv
    	bool IsConnected(); //checks if it's connected (this is a flag set durring a send/recv operation, and is not realtime)
    
        int ShowStatus(bool onlyreturn); //shows socket status and returns code
    	string GetError(bool clear); //gets error, and clears if specified
    	bool Connect(); //connects
		
		//send:
		
    	bool SendString(string data);
    	bool SendBitStream(BitStream data); //OBSOLETE - only here for backwards compatibility, use ByteStream
		bool SendByteStream(ByteStream data); 
    	
		
		
		//Receive:
		/**
		Appends data to "data" as long as it is received within timeout.  
		Otherwise returns false and does not append any data		
		*/
		bool RecvChar(char & data,uli timeout); //receive one char
		
    	bool Recv(BitStream & data,uli bytes,uli timeout);   //OBSOLETE - use ByteStream
    	bool Recv(ByteStream & data,uli bytes,uli timeout);   //append to ByteStream 
    	bool Recv(string & data,uli bytes,uli timeout); //append to String
		
    	bool RecvUntilChar(BitStream & data, char stopchar,uli timeout,uli max); //OBSOLETE - use ByteStream
		bool RecvUntilChar(ByteStream & data, char stopchar,uli timeout,uli max); 	//receive until specified char is gotten
    	bool RecvUntilChar(string & data, char stopchar,uli timeout,uli max); 	//receive data until specified char is gotten	
		
		void Disconnect(); 
    };
    
    




    //Server Stuff:
    //----------------------------------

	class SocketServer;
	class ClientInfo;
	
	//client class:
	//use this to talk to individual clients (the server instance will return an instance of this class)
	//This class acts as a "smart pointer" and will be removed automaticly when it is safe.  
	class ServerClient
	{
		private:
			ClientInfo * m_ci;
			bool m_null;		
			
			int RefCount(int c); //sets ref count of pointer, when this is 0 we delete the pointer			
			void Delete();       //this tells the server socket to free the memory
		
		public:
			ServerClient(ClientInfo * ci);
			ServerClient();
			~ServerClient();           //destructor!
			void Disconnect();         //this marks the client for disconnection			
			
			bool IsConnected();        //check if client is connected			
			bool Lock(bool);	       //locks client for read/write  (for thread safety)
			void Unlock();		       //unlocks for read/write		(for thread safety)
			unsigned long int QueueSize();          //returns queue size in bytes
			unsigned long int QueueSizeBits();      //returns queue size in bits
            string GetQueue();	                    //get data in queue
            bool PopData(string & data,int bytes);	//pop bytes as string - returns false if no data available
			bool PopBitStream(BitStream & data, int bytes);     //OBSOLETE - use ByteStream one below
			bool PopByteStream(ByteStream & data, int bytes);     //pop bytes as bytestream - returns false if no data available
            void TXdata(string bytes);              //send data TO CLIENT
            bool RXdata();                          //gets data, returns true if we got data, false if not, or if we DCed
            string GetIPstr(bool showport);         //get ip in string format (and port, optinal)
            BitStream GetIP(); 		                //get ip in bitstream format
			int GetPort();				            //get port in int form
			string GetPortstr();		            //get port in str form
			int GetFD();				            //returns file descriptor
			

			//overloads:
			ServerClient& operator=(const ServerClient&);
			ServerClient(const ServerClient & sc); //copy constructor (for "ServerClient a = b" situations)
			bool operator==(const ServerClient & comp); //assignment constructor			

	};
	
	
	
    
	//NOTE: This class for internal use only - should never be used outside rslib
    class ClientInfo
    {
        friend class SocketServer;
		friend class ServerClient;
        
        private:
            unsigned int m_RefCount; //used to keep track of references
			int RefCount(int);     //increase reference count (for "smart pointer" usability)
			
			ByteStream dataqueue;   //pending data
			SocketServer * server; //corresponding server class
			BitStream IpAddress;   //IP in bitstream format
			string IpAddressSTR;   //IP in str format
			int Port;			   //port in int format
			
			#ifdef rslib_pthread	
			MUTEX mutex;			//our mutex
			#endif
            bool connected;         //set to false to cause server to drop this connection
			bool disconnected;		//set to true if server has disconnected this (note, this is NOT direct opposite of connected)
			bool deleted; 			//set to true to free memory
           
			//RX buffer related stuff:			
			char * rxtmp;         //used as buffer for receiving data 
			int rxtmpsize;		//size of buffer
			int rxret;			//used as return value storage for rx
            int FD;                 //descriptor
          
            ClientInfo(SocketServer * pserver,int pFD,sockaddr_in paddr);   
			~ClientInfo();   
			bool Lock(bool);	//locks client for read/write  (for thread safety)
			void Unlock();		//unlocks for read/write		(for thread safety)			
			void SetRxBufferSize(int bytes);	//changes the size of the RX buffer
            unsigned long int QueueSize();	//get queue size
			unsigned long int QueueSizeBits();//get queue size in bits
			void Disconnect();	//force disconnect a client
            bool IsConnected(); //check if client is connected
            bool RXdata();          //gets data, returns true if we got data, false if not, or if we DCed	
            string GetIPstr(bool showport); //get ip in string format (and port, optinal)
            BitStream GetIP(); 		//get ip in bitstream format
			int GetPort();				//get port in int form
			string GetPortstr();		//get port in str form
    };
    


	//server class
	//use this to declare a new server/listener
    class SocketServer
    {
        friend class ClientInfo;
		
		private:
			xVector<ClientInfo*> Clients;	  //current clients in session
            xVector<ClientInfo*> NewClients; //put clients here when they connect
			xVector<string> DCedClients;//string value of ip:port for DCed clients
			
			#ifdef rslib_pthread
			MUTEX mutex;
			#endif

            string Message;
			bool listening;
            int BackLog;
            int sockfd;
            struct sockaddr_in my_addr;
			int defaultbuffersize;

            void SetMessage(string msg,bool error);  
            
            void ClientAdd(ClientInfo *);
			void ClientDisconnected(ClientInfo *);
            void ClientDelete(ClientInfo *);
                 
			int nextpollclient;
        
        public:
			bool TrackClientDC; //set to true to keep track of disconnected clients (off by default)
		
            SocketServer();
            ~SocketServer();
			bool debugmode;
			bool Lock(bool block);	//lock for thread safety
			void Unlock();			//unlock for thread safety
            string GetMessage();
            int GetConnectionNum();
            void SetBacklog(int);          
			void SetBufferSize(int psize);//set default buffer size 			
            bool Listen(int port,string ip);//Sets listen parameters
            bool Poll(int pooltimeout);//poll for any new connections (also calls PollData(true) which checks for connection status)
            void PollData(bool connectionsonly);//Poll all clients for data/disconnections (can also just call RXdata for each client)
			int GetConNum(); //return number of connections            
            void Shutdown(bool deallocate); //shuts down listen socket
            
            xVector<ServerClient> GetConnectedClients(bool onlywithdata);    //returns vector of pointers for all clients
			bool GetNewClient(ServerClient & client); //gets a new client (returns true if there is one)
			bool GetDCedClient(string & dced);	//returns xVector containing strings of DCed clients
			void DeleteDeadClients(); 	//deletes all disconnected clients - warning: any existing ServerClient objects could cause crashes if still in scope
			static void SetFDlimit(int limit);	//sets system file descriptor limit (by default its 1024 which may be too low for some server systems)
        
    };
	
	
	//static functions
	bool ResolveHost(string host,BitStream & ip);  	//resolves a host name
	bool ValidateIP(string ip); 					//validates that the string is a valid IP address (ex: 4 octets, 0-255)
	bool ValidateHostName(string hostname,bool requireresolve,bool acceptip); //validates that a hostname is valid 
	
	
	
	

}   


#endif 
