/**********************************************************
EnviroState C++ source file
Environmental Monitoring System
Last modified by Red Squirrel on Oct-12-2012 12:57:22am
Filepath: includes/Core/AgentCore.cpp
Lines of code: 296

Description: 

***********************************************************/


AgentCore::AgentCore(ClArgs args,AppLogger * logger)
{
	m_args=args;
	m_logger=logger;
	m_config=NULL;
	m_tcpclient=NULL;
	m_serverdown=false;
	
	m_lastdatasend=0;
	m_lastconattempt=0;
	m_lastpidcheck=time(NULL);
	m_lastmonret=0;
}

AgentCore::~AgentCore()
{
	if(m_config!=NULL)delete m_config;
	Disconnect();
	ClearMonList();
}


bool AgentCore::TcpSend(string tosend)
{
	Log(5,"TcpSend: " + tosend);
	
	if(!IsConnected())
	{
		Log(0,"Cannot send data, not connected");
		return false;
	}
	
	
	return m_tcpclient->SendString(tosend + "\r\n");
}


string AgentCore::TcpRecv()
{
	if(!IsConnected())
	{
		Log(0,"Cannot recv data, not connected");
		return false;
	}	
	
	string ret;	
	m_tcpclient->RecvUntilChar(ret, '\r',5000,1024);
	
	if(ret=="")
	{
		Log(5,"TcpRecv: [empty]");
		return "";	
	}
	
	//there is still a \n remaining, fetch it to clear it out:
	char tmpchar;
	m_tcpclient->RecvChar(tmpchar,1000);
	
	ret.erase(ret.size(),1);//clear \r
	
	Log(5,"TcpRecv: " + HexDump(ret));
	return ret;
}



void AgentCore::Log(int level,string text)
{
	if(m_logger!=NULL)m_logger->Log(level,m_agentname + ": " + text);	
}

bool AgentCore::Load()
{
	string configpath=m_args.GetValue("confpath")!="" ? m_args.GetValue("confpath") : "/etc/envirostate/";  
	if(configpath[configpath.size()-1]!='/')configpath.append(1,'/'); //ensure it has trailing slash

	
	//start loading config
	
	Log(2,"Loading AGENT config from directory: " + configpath);
	
	m_config = new MlConfFile(configpath + "agent.conf",m_logger);	
	m_config->DebugLevel=5;	
	
	
	Log(5,"Entire config file schema: " + m_config->DumpData());
	
	if(!m_config->Load())
	{
		Log(0,"Fatal: Unable to load config!  There may be syntax errors, or it does not exist.  Run -genconf to generate a basic sample config structure");
		
		return false;
	}
	
	
	//check for config parameters:
	
	if(!m_config->ValueExists("agentname"))
	{
		Log(0,"agentname does not exist, exiting");
		return false;
	}
	else
	{
		m_agentname = m_config->GetValueStr("agentname");
	}
	
	
	if(!m_config->ValueExists("serverhost"))
	{
		Log(0,"serverhost does not exist, exiting");
		return false;
	}
	else
	{
		m_serverhost = m_config->GetValueStr("serverhost");
	}
	
	
	if(!m_config->ValueExists("serverport"))
	{
		Log(0,"serverport does not exist, exiting");
		return false;
	}
	else
	{
		m_serverport = m_config->GetValueInt("serverport");
	}
	
	
	if(!m_config->ValueExists("datasendinterval"))
	{
		Log(0,"datasendinterval does not exist, setting to default of 300");
		m_datasendinterval=300;
	}
	else
	{
		m_datasendinterval = m_config->GetValueInt("datasendinterval");
	}
	


	
	return true;
}

void AgentCore::Terminate()
{
	Log(3,"Terminating: doing some cleanup first...");	
	
	SendMonitors();
	Disconnect();
}



bool AgentCore::Poll()
{
	Log(6,"AgentCore::Poll()");	
	
	//check pid: hard coded rate is 10 secs
	if((m_lastpidcheck+10)<time(NULL))
	{
		m_lastpidcheck=time(NULL);		
		Log(6,"AgentCore::Poll() check pid file...");	
		
		fstream fin("envirostate-agent.pid",ios::in);
		bool terminate=false;
		if(!fin)terminate=true;
		fin.close();
		
		if(terminate)
		{
			Log(6,"pid file deleted, terminating app gracefully.");	
			
			Terminate();
			
			return false;
		}
	}
	
	
	//check if we need to send data to server:
	if((m_lastdatasend+m_datasendinterval)<time(NULL) && !(m_serverdown && (m_lastconattempt+15)>time(NULL)))
	{		
		Log(4,"AgentCore::Poll() send data to server");
				
		if(SendMonitors())
		{
			m_lastdatasend=time(NULL);
		}
		else
		{
			Log(0,"AgentCore::Poll() Error sending data to server");
		}
		
		//check for new/updated monitor data:  This is checked at a rate 10x data send rate (Ex: if data send is every 5 minutes then this happens every 50min)
		if((m_lastmonret+(m_datasendinterval*10))<time(NULL))
		{			
			Log(4,"AgentCore::Poll() check for new/updated monitors...");			
			
			ClearMonList();
			
			if(RecvMonitors())
			{
				m_lastmonret=time(NULL);	
			}
			else
			{
				Log(0,"AgentCore::Poll() Error checking for new/updated monitors");
			}
		}		
	}
	

	
	//poll monitors:
	
	MonitorEntry * entry=NULL;
	for(int i=0;m_monitorentries.Get(entry,i);i++)
	{
		entry->Poll();
	}

	
	rslib::Wait(100000);	
	return true;
}


bool AgentCore::CheckConnection()
{
	if(IsConnected())return true;
	
	if(m_serverdown && (m_lastconattempt+15)>time(NULL))return false;		
	
	return Connect();
}



bool AgentCore::Connect()
{
	if(m_tcpclient==NULL)m_tcpclient = new SocketClient(m_serverhost,m_serverport);	
	if(m_tcpclient->IsConnected())m_tcpclient->Disconnect();	
	
	Log(1,"Connecting to server [" + m_serverhost + ":" + BitStream::Int2Str(m_serverport) + "]");	
	
	if(!m_tcpclient->Connect())
	{
		Log(0,"Error connecting, will try again later");		
		m_lastconattempt=time(NULL);
		m_serverdown=true;
	}
	
	Log(2,"Connected to server");
	
	//initial recv (version info)
	TcpRecv();
	
	
	//tell server to load proper agent (this one) :
	
	TcpSend("lagt");			
	if(TcpRecv()!="ok")
	{
		Log(0,"unexpected response from server");
		return false;
	}	
	
	
	TcpSend(m_agentname);
	if(TcpRecv()!="ok")
	{
		Log(0,"unexpected response from server");
		return false;
	}
	
	m_serverdown=false;
	return true;
}

void AgentCore::Disconnect()
{
	if(m_tcpclient!=NULL)
	{
		m_tcpclient->Disconnect();
		delete m_tcpclient;
		m_tcpclient=NULL;
		
		Log(1,"Disconnected from server");	
	}	
}

bool AgentCore::IsConnected()
{
	if(m_tcpclient==NULL)return false;
	
	return m_tcpclient->IsConnected();
}


void AgentCore::ClearMonList()
{
	MonitorEntry * monitor=NULL;
	while(m_monitorentries.PopFrom(monitor,0))
	{
		delete monitor;
	}
}



bool AgentCore::RecvMonitors()
{
	if(!CheckConnection())return false;
	
	
	Log(3,"Requesting monitor list...");	

	
	TcpSend("mons");
	if(TcpRecv()!="ok")
	{
		Log(0,"unexpected response from server");
		return false;
	}
	
	
	//load monitors:
	Log(4,"Start monitor load loop");
	while(1)
	{
		TcpSend("n");		
		if(TcpRecv()!="i")break;	
		
		string monname=TcpRecv();
		int monpollinterval=atoi(TcpRecv().c_str());
		string monsource=TcpRecv();		
		float monmultiplier=atof(TcpRecv().c_str());		
		
		MonitorEntry * entry = new MonitorEntry(this,monname);
		m_monitorentries.Add(entry);
		
		entry->m_pollinterval=monpollinterval;
		entry->m_source=monsource;
		entry->m_multiplier=monmultiplier;
	}
	Log(4,"End monitor load loop");
	
	

	Disconnect();
	
	return true;
}


bool AgentCore::SendMonitors()	
{
	if(!CheckConnection())return false;

	
	MonitorEntry * monitor=NULL;
	DataEntry * dataentry=NULL;
	for(int i=0;m_monitorentries.Get(monitor,i);i++)
	{
		TcpSend("lmon");	
		if(TcpRecv()!="ok")
		{
			Log(0,"Unexpected response from server at lmon");
			return false;
		}
		
		TcpSend(monitor->m_name);	
		if(TcpRecv()!="ok")
		{
			Log(0,"Unexpected response from server at lmon: name");
			return false;
		}
		
		while(monitor->m_dataentries.Get(dataentry,0))
		{
			TcpSend("wval");
			if(TcpRecv()!="ok")
			{
				Log(0,"Unexpected response from server at lmon: name");
				return false;
			}
			
			TcpSend(BitStream::Int2Str(dataentry->value));
			TcpSend(BitStream::Int2Str(dataentry->date));
			
			if(TcpRecv()!="ok")
			{
				Log(0,"Unexpected response from server at lmon: name");
				return false;
			}
			else
			{
				monitor->m_dataentries.Erase(dataentry);
				delete dataentry;
			}
		}
		
		TcpSend("wfin");
		if(TcpRecv()!="ok")
		{
			Log(0,"Unexpected response from server at wfin");
			return false;
		}
		
	}
	
	Disconnect();
	
	return true;
}