/**********************************************************
EnviroState C++ source file
Environmental Monitoring System
Last modified by Red Squirrel on Oct-04-2012 01:34:51am
Filepath: includes/Core/ServerCore.cpp
Lines of code: 184

Description: 

***********************************************************/


ServerCore::ServerCore(ClArgs args,AppLogger * logger)
{
    m_args=args;
    m_logger=logger;
	m_config=NULL;
	m_defaultmonitor=NULL;
	m_server=true;
	m_lastdatadump=DateTime::Now();
	m_lastpidcheck=time(NULL);
}

ServerCore::~ServerCore()
{
	if(m_defaultmonitor!=NULL)delete m_defaultmonitor;
	if(m_config!=NULL)delete m_config;	
	

	m_serverclass.Shutdown(true);		
	
	//delete agents:
	Agent * tmpagent;
	while(m_agents.PopFrom(tmpagent,0))
	{
		delete tmpagent;
	}
	
	
	//delete netsessions
	AgentNetSession * tmpnetsession=NULL;
	while(m_agentsessions.PopFrom(tmpnetsession,0))
	{
		delete tmpnetsession;
	}		
	
	
}


void ServerCore::Log(int level,string text)
{
	if(m_logger!=NULL)m_logger->Log(level,text);	
}





bool ServerCore::Load()
{
	string configpath=m_args.GetValue("confpath")!="" ? m_args.GetValue("confpath") : "/etc/envirostate/";  
	if(configpath[configpath.size()-1]!='/')configpath.append(1,'/'); //ensure it has trailing slash


	
	
	//start loading config

	Log(2,"Loading SERVER config from directory: " + configpath);
	
	m_config = new MlConfFile(configpath + "server.conf",m_logger);
	
	m_config->DebugLevel=m_args.GetValueInt("confdebuglevel");
	
	if(!m_config->Load())
	{
		Log(0,"Fatal: Unable to load config!  There may be syntax errors, or it does not exist.  Run -genconf to generate a basic sample config structure");
		
		return false;
	}	
	
	
	
	Log(5,"Entire config file schema: \n" + m_config->DumpData());
	
		
	
	//now that we read the config file, ensure that it has all the right parameters, set any defaults, etc.
	
	Log(2,"Verifying config...");
	

	
	if(!m_config->ValueExists("datadumpinterval"))
	{
		Log(0,"datadumpinterval does not exist, setting default to 300s");
		m_datadumpinterval=10;
	}
	else
	{
		m_datadumpinterval = m_config->GetValueInt("datadumpinterval");
	}
	
			
	if(!m_config->ValueExists("listenport"))
	{
		Log(0,"listenport does not exist, setting default to 9000");
		m_listenport=9000;
	}
	else
	{
		m_listenport = m_config->GetValueInt("listenport");
	}		
											
	
	
	
	if(!m_config->BlockExists("agents"))
	{
		Log(0,"\"agents\" block does not exist.");
		return false;
	}
	
	if(!m_config->BlockExists("defaultmonitor"))
	{
		Log(0,"\"defaultmonitors\" block does not exist.");
		return false;
	}	
	
	
	
	//now that we determined that the variables we want exist, we start loading up the agents and monitors
	
	Log(2,"Loading Default Monitor...");
	
	
	//start with default, we will copy new monitors from this one
	
	Log(3,"Monitor: defaultmonitor...");
	Monitor * tmpdefault = new Monitor(this,NULL,"defaultmonitor"); 
	tmpdefault->Load("",m_config);		       			            
	m_defaultmonitor=tmpdefault;									

	
	
	//load rest of agent/monitors
	
	
	Log(3,"Loading agents...");
	
	
	
	xVector<string> agentsstr = m_config->GetBlocks("agents");
	
	string tmpstr;
	Agent * tmpagent;
	
	for(int i=0;agentsstr.Get(tmpstr,i);i++)
	{			
		if(m_config->GetValueInt("agents." + tmpstr + ".enabled")==1)
		{	
			Log(3,"Agent: " + tmpstr + "...");	
			tmpagent = new Agent(this,tmpstr);
			tmpagent->Load();
			m_agents.Add(tmpagent);	
		}
		else
		{
			Log(3,"Agent: " + tmpstr + "...Not loading - disabled");	
		}
	}	
	
	
	
	Log(2,"Starting server...");
	
	if(!m_serverclass.Listen(m_listenport,""))
	{
		Log(0,"Error starting server, shutting down");
		return false;
	}


	
	
	Log(3,"Done loading core");
	return true;
}



void ServerCore::Terminate()
{
	Log(3,"ServerCore::Terminate() - ensuring all agents dump their data to disk...");
	Agent * tmpagent;
	for(int i=0;m_agents.Get(tmpagent,i);i++)
	{
		tmpagent->Dump2Disk();
	}
	
	
	
	Log(3,"ServerCore::Terminate() done");
}




bool ServerCore::Poll()
{
	Log(6,"ServerCore::Poll()");	
	bool sleep=true; //sleep at end of poll interval to avoid hogging cpu
	
	
	if((m_lastpidcheck+10)<time(NULL))
	{
		m_lastpidcheck=time(NULL);
		
		Log(6,"ServerCore::Poll() check pid file...");	
		
		fstream fin("envirostate-server.pid",ios::in);
		bool terminate=false;
		if(!fin)terminate=true;
		fin.close();
		
		if(terminate)
		{
			Log(6,"pid file deleted, terminating app gracefully.");	
			
			Terminate();
			
			return false;
		}
	}
	

	Log(6,"Poll tcp connections...");		
	
	m_serverclass.Poll();		//poll for connections
	m_serverclass.PollData(true);  //poll existing connections
	
	
	//check for new connections and put them in our own custom class:
	ServerClient client;
	if(m_serverclass.GetNewClient(client))
	{
		m_agentsessions.Add(new AgentNetSession(this,client));
	}
	
	//check for disconnected tcp clients so we can clean up
	AgentNetSession * tmpnetsession=NULL;
	for(int i=0;m_agentsessions.Get(tmpnetsession,i);i++)
	{
		if(tmpnetsession->Disconnected())
		{
			m_agentsessions.Erase(tmpnetsession);
			delete tmpnetsession;
			tmpnetsession=NULL;
		}
		else
		{				
			tmpnetsession->Poll();			
		}
	}
	

	
	DateTime lastplus = m_lastdatadump;
	lastplus.AddSeconds(m_datadumpinterval);	
	
	bool dump2disk = false;
	
	if(DateTime::Now()>lastplus)
	{
		Log(3,"Monitor data will be written to disk next poll");
		mkdir("output",S_IRWXU);
		dump2disk=true;
		m_lastdatadump=DateTime::Now();
	}
	
	
	Log(6,"Poll agents...");
	
	Agent * tmpagent;
	for(int i=0;m_agents.Get(tmpagent,i);i++)
	{
		if(dump2disk)tmpagent->DoDump2Disk=true;
		tmpagent->Poll();			
	}
		
	
	
	if(sleep)rslib::Wait(5000);
	
	return true;
}





Agent * ServerCore::GetAgent(string name)
{
	
	Agent * tmpagent=NULL;
	for(int i=0;m_agents.Get(tmpagent,i);i++)
	{
		if(tmpagent->m_name==name)return tmpagent;
	}
	
	return NULL;	
}


